@@ -1,3 +1,5 @@
+# shadowstep/element/element.py
+
 import inspect
 import logging
 import time
@@ -19,6 +21,7 @@ from selenium.webdriver.remote.shadowroot import ShadowRoot
 from selenium.webdriver.support.wait import WebDriverWait
 
 from shadowstep.element.base import ElementBase
+from shadowstep.elements.elements import Elements
 from shadowstep.utils import conditions
 from shadowstep.utils.utils import find_coordinates_by_vector
 
@@ -40,12 +43,13 @@ class GeneralElementException(WebDriverException):
 class Element(ElementBase):
     """
     A class to represent a UI element in the Shadowstep application.
-    !WARNING! USE XPATH STRATEGY ONLY !WARNING!
+    !WARNING! TUPLE LOCATOR USE XPATH STRATEGY ONLY !WARNING!
+    Please use dict locator
     """
 
     def __init__(self,
                  locator: Union[Tuple, Dict[str, str], 'Element'] = None,
-                 base=None,
+                 base: 'Shadowstep' = None,
                  timeout: int = 30,
                  poll_frequency: float = 0.5,
                  ignored_exceptions: typing.Optional[WaitExcTypes] = None,
@@ -104,98 +108,47 @@ class Element(ElementBase):
             self,
             locator: Union[Tuple, Dict[str, str], 'Element'],
             contains: bool = False,
-            max_count: int = 10
-    ) -> typing.Generator['Element', None, None]:
+            max_count: int = 100
+    ) -> Elements:
         self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
+
         if isinstance(locator, Element):
             locator = locator.locator
-        resolved_locator = self.handle_locator(locator, contains)
-        base_xpath = self._get_xpath()
 
+        base_xpath = self._get_xpath()
         if not base_xpath:
             raise GeneralElementException("Unable to resolve base xpath")
 
-        # Убираем финальные / у base_xpath и начальные у локатора
-        base_xpath = base_xpath.rstrip('/')
-        child_xpath = resolved_locator[1].lstrip('/')
+        resolved_xpath = self.locator_converter.to_xpath(locator)[1]
 
-        for index in range(1, max_count + 1):
-            try:
-                element = Element(
-                    locator=('xpath', f"{base_xpath}//{child_xpath}[{index}]"),
-                    base=self.base,
-                    timeout=self.timeout,
-                    poll_frequency=self.poll_frequency,
-                    ignored_exceptions=self.ignored_exceptions,
-                    contains=contains
-                )
-                # Пробуем получить базовый атрибут
-                if element.get_attribute("class") is None:
-                    break
-                yield element
-            except NoSuchElementException:
-                break
-            except WebDriverException:
-                break
+        def generator() -> typing.Iterator['Element']:
+            missed = 0
+            for index in range(1, max_count + 1):
+                indexed_xpath = f"({resolved_xpath})[{index}]"
+                try:
+                    element = Element(
+                        locator=("xpath", indexed_xpath),
+                        base=self.base,
+                        timeout=3,
+                        poll_frequency=self.poll_frequency,
+                        ignored_exceptions=self.ignored_exceptions,
+                        contains=contains
+                    )
+                except (NoSuchElementException, WebDriverException):
+                    missed += 1
+                    if missed >= 2:
+                        break
+                    continue
+                try:
+                    _ = element.get_attribute("class")
+                    yield element
+                    missed = 0
+                except Exception:
+                    missed += 1
+                    if missed >= 2:
+                        break
+        return Elements(generator)
 
-    def get_elements_greedy(
-            self,
-            locator: Union[Tuple, Dict[str, str], 'Element'],
-            contains: bool = False,
-            timeout: int = 30,
-    ) -> typing.List['Element']:
-        """
-        Returns a list of elements found using find_elements.
-        This is a greedy method and fetches all matching elements immediately.
-
-        Args:
-            locator: Locator to search for child elements.
-            contains: If True, performs partial match.
-            timeout: Timeout to wait for elements.
-
-        Returns:
-            List of Element instances matching the locator.
-        """
-        self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
-        try:
-            self._get_driver()
-            if isinstance(locator, Element):
-                locator = locator.locator
-            resolved_locator = self.handle_locator(locator, contains)
-            base_xpath = self._get_xpath()
-
-            if not base_xpath:
-                raise GeneralElementException("Unable to resolve base xpath")
-
-            base_xpath = base_xpath.rstrip('/')
-            child_xpath = resolved_locator[1].lstrip('/')
-
-            # Совмещённый XPath для поиска потомков
-            full_xpath = f"{base_xpath}//{child_xpath}"
-
-            base_element = self._get_element(locator=self.locator)
-
-            found_elements = base_element.find_elements('xpath', full_xpath)
-
-            result = []
-            for i in range(len(found_elements)):
-                xpath = f"{full_xpath}[{i + 1}]"
-                result.append(Element(
-                    locator=('xpath', xpath),
-                    base=self.base,
-                    timeout=timeout,
-                    poll_frequency=self.poll_frequency,
-                    ignored_exceptions=self.ignored_exceptions,
-                    contains=contains
-                ))
-
-            return result
-        except NoSuchDriverException as error:
-            self._handle_driver_error(error)
-            return []
-        except InvalidSessionIdException as error:
-            self._handle_driver_error(error)
-            return []
 
     def get_attributes(self) -> Optional[Dict[str, str]]:
         """Fetch all XML attributes of the element by matching locator against page source.
@@ -724,22 +677,6 @@ class Element(ElementBase):
             stacktrace=traceback.format_stack()
         )
 
-    def is_not_within_screen(self) -> bool:
-        """Checks whether the element is not within the visible screen bounds.
-
-        Returns:
-            bool: True if element is not displayed or outside screen bounds.
-        """
-        self.logger.debug(f"{inspect.currentframe().f_code.co_name}")
-        try:
-            return not self.is_visible()
-        except GeneralElementException as error:
-            self.logger.warning(f"is_not_within_screen fallback due to: {error}")
-            return True
-        except Exception as error:
-            self.logger.warning(f"is_not_within_screen unexpected error: {error}")
-            return True
-
     def is_selected(self) -> bool:
         """Returns whether the element is selected.
 
@@ -2333,3 +2270,10 @@ class Element(ElementBase):
             return True
         except TimeoutException:
             return False
+
+    @property
+    def should(self) -> 'Should':
+        """Provides DSL-like assertions: element.should.have.text(...), etc."""
+        from shadowstep.element.should import Should  # импорт внутри метода для избежания циклической зависимости
+        return Should(self)
+
